# pylint:disable=too-many-arguments

from unittest import TestCase
from unittest.mock import patch, mock_open, call

from io import BytesIO

import json

from har_extractor import (
    format_size, get_unused_name, write, get_out_dir,
    format_entry, get_entry_content, get_entry_path,
    get_entries, dirnames, move_files_to_dir, make_entry_dirs
)

from data import TEST_ARCHIVE


class TestUtils(TestCase):
    def test_format_size(self):
        self.assertEqual(format_size(-1), '<invalid size>')
        self.assertEqual(format_size(0), '0B')
        self.assertEqual(format_size(512), '512B')
        self.assertEqual(format_size(1024), '1K')
        self.assertEqual(format_size(2048), '2K')
        self.assertEqual(format_size(1536), '1.50K')
        self.assertEqual(format_size(4 * (1024 ** 2)), '4M')
        self.assertEqual(format_size(4 * (1024 ** 3) + 1), '4.00G')
        self.assertEqual(format_size(1024 ** 4), '1T')

    @staticmethod
    @patch('builtins.open', new_callable=mock_open)
    def test_write(open_mock):
        handle = open_mock()
        write('content', 'fname')
        open_mock.assert_called_with('fname', 'w')
        handle.write.assert_called_with('content')
        write(b'\x01\x02\x03', '')
        open_mock.assert_called_with('', 'wb')
        handle.write.assert_called_with(b'\x01\x02\x03')

    @patch('os.path.exists')
    def test_get_unused_name(self, exists):
        exists.side_effect = [False]
        self.assertEqual(get_unused_name('/dir/name.ext'), '/dir/name.ext')
        exists.side_effect = [True, False]
        self.assertEqual(get_unused_name('/dir/name.ext'), '/dir/name.1.ext')
        exists.side_effect = [True, True, True, False]
        self.assertEqual(get_unused_name('/dir/name.ext'), '/dir/name.3.ext')

    @patch('os.path.exists', return_value=False)
    @patch('os.path.isdir', return_value=True)
    def test_get_out_dir(self, isdir, exists):
        self.assertEqual(get_out_dir(None, 'default'), 'default')
        self.assertEqual(get_out_dir('', 'default'), 'default')
        self.assertEqual(exists.call_count, 0)
        self.assertEqual(isdir.call_count, 0)

        self.assertEqual(get_out_dir('path', 'default'), 'path')
        exists.assert_called_with('path')
        exists.reset_mock()
        self.assertEqual(isdir.call_count, 0)

        exists.return_value = True
        self.assertEqual(get_out_dir('path', 'default'), 'path/default')
        exists.assert_called_with('path')
        isdir.assert_called_with('path')

        with self.assertRaises(ValueError):
            isdir.return_value = False
            get_out_dir('path', 'default')

    @patch('har_extractor.format_size', return_value='<size>')
    def test_format_entry(self, format_size_mock):
        entry = {
            'request': {
                'method': 'METHOD',
                'url': '/url'
            },
            'response': {
                'status': 123,
                'statusText': 'Status Text',
                'content': {
                    'mimeType': 'mime/type',
                    'size': 321
                }
            }
        }
        formatted = 'METHOD /url -> 123 Status Text mime/type <size>'
        self.assertEqual(format_entry(entry), formatted)
        format_size_mock.assert_called_with(321)

    def test_format_entry_invalid(self):
        formatted = '<no method> <no url> -> <no status>' \
                    ' <no status text> <no mime type> <invalid size>'
        self.assertEqual(format_entry({}), formatted)

    def test_get_entry_path(self):
        test = lambda url: get_entry_path({'request': {'url': url}})
        self.assertEqual(test('http://127.0.0.1'), 'index.html')
        self.assertEqual(test('http://127.0.0.1/'), 'index.html')
        self.assertEqual(test('http://127.0.0.1/dir/'), 'dir')
        self.assertEqual(test('http://127.0.0.1/dir'), 'dir')
        self.assertEqual(test('http://127.0.0.1/dir/name'), 'name')
        self.assertEqual(test('http://127.0.0.1/dir/name/'), 'name')
        self.assertEqual(test('http://127.0.0.1/dir/name?arg'), 'name')
        self.assertEqual(test('http://127.0.0.1/dir/name/?arg'), 'name')
        with self.assertRaises(ValueError):
            get_entry_path({})

    def test_get_entry_path_subdirs(self):
        test = lambda url: get_entry_path({'request': {'url': url}}, True)
        self.assertEqual(test('http://127.0.0.1'), '127.0.0.1/index.html')
        self.assertEqual(test('http://127.0.0.1/'), '127.0.0.1/index.html')
        self.assertEqual(test('http://127.0.0.1/dir/'), '127.0.0.1/dir')
        self.assertEqual(test('http://127.0.0.1/dir'), '127.0.0.1/dir')
        self.assertEqual(test('http://127.0.0.1/dir/name'), '127.0.0.1/dir/name')
        self.assertEqual(test('http://127.0.0.1/dir/name/'), '127.0.0.1/dir/name')
        self.assertEqual(test('http://127.0.0.1/dir/name?arg'), '127.0.0.1/dir/name')
        self.assertEqual(test('http://127.0.0.1/dir/name/?arg'), '127.0.0.1/dir/name')

    def test_get_entry_content(self):
        entry = {
            'response': {
                'content': {
                }
            }
        }
        self.assertIsNone(get_entry_content({}))
        self.assertIsNone(get_entry_content(entry))

        entry['response']['content']['text'] = ''
        self.assertIsNone(get_entry_content(entry))

        entry['response']['content']['text'] = 'test'
        self.assertEqual(get_entry_content(entry), 'test')

        entry['response']['content']['text'] = 'dGVzdA=='
        entry['response']['content']['encoding'] = 'base64'
        self.assertEqual(get_entry_content(entry), b'test')

        with self.assertRaises(ValueError):
            entry['response']['content']['encoding'] = 'encoding'
            get_entry_content(entry)

    def test_dirnames(self):
        self.assertEqual(dirnames('/x/y/z/', '/x/'), ['/x/y'])
        self.assertEqual(dirnames('/x/y/z/', '/x'), ['/x/y'])
        self.assertEqual(dirnames('/x/y/z', '/x'), ['/x/y'])
        self.assertEqual(dirnames('/x/y/z/u/v', '/x/'),
                         ['/x/y/z/u', '/x/y/z', '/x/y'])

    @patch('shutil.move')
    @patch('os.path.exists', side_effect=[True, True, False])
    def test_move_files_to_dir(self, exists, move):
        move_files_to_dir('/a/b/c.d', '/a/b/c.3.d')
        exists.assert_has_calls([
            call('/a/b/c.%d.d' % i) for i in range(1, 4)
        ])
        self.assertEqual(exists.call_count, 3)
        move.assert_has_calls([
            call('/a/b/c.3.d', '/a/b/c.d/index.html'),
            call('/a/b/c.1.d', '/a/b/c.d/index.1.html'),
            call('/a/b/c.2.d', '/a/b/c.d/index.2.html')
        ])
        self.assertEqual(move.call_count, 3)

    @patch('har_extractor.move_files_to_dir')
    @patch('shutil.move')
    @patch('os.makedirs')
    @patch('os.mkdir')
    def test_make_entry_dirs(self, mkdir, mkdirs, move, move_files_to_dir_):
        make_entry_dirs('/root', '/root/dir/entry')
        mkdirs.assert_called_with('/root/dir', exist_ok=True)
        self.assertEqual(mkdirs.call_count, 1)
        self.assertEqual(mkdir.call_count, 0)
        self.assertEqual(move.call_count, 0)
        self.assertEqual(move_files_to_dir_.call_count, 0)

    @patch('har_extractor.get_unused_name', return_value='unused')
    @patch('har_extractor.move_files_to_dir')
    @patch('shutil.move')
    @patch('os.makedirs', side_effect=OSError)
    @patch('os.mkdir')
    @patch('os.path.exists', side_effect=[False, True, True])
    @patch('os.path.isdir', side_effect=[False, True])
    def test_make_entry_dirs_error(self, isdir, exists,
                                   mkdir, mkdirs, move,
                                   move_files_to_dir_, get_unused_name_):
        make_entry_dirs('/root/dir', '/root/dir/dir2/dir3/4/entry')

        mkdirs.assert_called_with('/root/dir/dir2/dir3/4', exist_ok=True)
        self.assertEqual(mkdirs.call_count, 1)

        exists.assert_has_calls([
            call('/root/dir/dir2'),
            call('/root/dir/dir2/dir3'),
            call('/root/dir/dir2/dir3/4')
        ])
        self.assertEqual(exists.call_count, 3)

        isdir.assert_has_calls([
            call('/root/dir/dir2/dir3'),
            call('/root/dir/dir2/dir3/4')
        ])
        self.assertEqual(isdir.call_count, 2)

        mkdir.assert_has_calls([
            call('/root/dir/dir2'),
            call('/root/dir/dir2/dir3')
        ])
        self.assertEqual(mkdir.call_count, 2)

        get_unused_name_.assert_called_with('/root/dir/dir2/dir3')
        self.assertEqual(get_unused_name_.call_count, 1)

        move.assert_called_with('/root/dir/dir2/dir3', 'unused')
        self.assertEqual(move.call_count, 1)

        move_files_to_dir_.assert_called_with('/root/dir/dir2/dir3', 'unused')
        self.assertEqual(move_files_to_dir_.call_count, 1)


class TestGetEntries(TestCase):
    def test_iterative(self):
        fp = BytesIO(json.dumps(TEST_ARCHIVE).encode('utf-8'))
        items = get_entries(fp, True)
        self.assertNotIsInstance(items, list)
        self.assertEqual(list(items), TEST_ARCHIVE['log']['entries'])

    def test_not_iterative(self):
        fp = BytesIO(json.dumps(TEST_ARCHIVE).encode('utf-8'))
        items = get_entries(fp, False)
        self.assertIsInstance(items, list)
        self.assertEqual(items, TEST_ARCHIVE['log']['entries'])

    @patch('sys.stdin')
    def test_stdin(self, stdin):
        stdin.buffer = BytesIO(json.dumps(TEST_ARCHIVE).encode('utf-8'))
        items = get_entries(stdin, False)
        self.assertNotIsInstance(items, list)
        self.assertEqual(list(items), TEST_ARCHIVE['log']['entries'])

    @patch('har_extractor.ijson', new=None)
    def test_no_ijson(self):
        fp = BytesIO(json.dumps(TEST_ARCHIVE).encode('utf-8'))
        items = get_entries(fp, True)
        self.assertIsInstance(items, list)
        self.assertEqual(items, TEST_ARCHIVE['log']['entries'])



# pylint:disable=too-many-arguments

from unittest import TestCase
from unittest.mock import patch, call
from io import StringIO

import os

from har_extractor import extract

from data import (
    TEST_ARCHIVE, TEST_ARCHIVE_LIST, TEST_ARCHIVE_CONTENTS,
    TEST_ARCHIVE_VERBOSE, TEST_ARCHIVE_INVALID_VERBOSE,
    TEST_ARCHIVE_INVALID, TEST_ARCHIVE_INVALID_LIST
)

@patch('os.path.exists', return_value=False)
@patch('os.makedirs')
@patch('har_extractor.make_entry_dirs')
@patch('har_extractor.write')
@patch('sys.stderr', new_callable=StringIO)
@patch('sys.stdout', new_callable=StringIO)
class TestDoExtract(TestCase):
    def test_extract(self, stdout, stderr,
                     write, make_entry_dirs, makedirs, _):
        extract(TEST_ARCHIVE['log']['entries'], 'dir/')
        stdout.seek(0)
        stderr.seek(0)
        makedirs.assert_called_with('dir/', exist_ok=True)
        write.assert_has_calls([
            call(content, os.path.join('dir', fname))
            for content, fname in TEST_ARCHIVE_CONTENTS
        ])
        self.assertEqual(make_entry_dirs.call_count, 0)
        self.assertEqual(stdout.read(), '')
        self.assertEqual(stderr.read(), '')

    def test_extract_subdirs(self, stdout, stderr,
                             write, make_entry_dirs, makedirs, _):
        extract(TEST_ARCHIVE['log']['entries'], 'dir/', subdirs=True)
        stdout.seek(0)
        stderr.seek(0)
        makedirs.assert_has_calles([
            call('dir/', exist_ok=True),
            call('dir/127.0.0.1', exist_ok=True)
        ])
        write.assert_has_calls([
            call(content, os.path.join('dir/127.0.0.1', fname))
            for content, fname in TEST_ARCHIVE_CONTENTS
        ])
        make_entry_dirs.assert_has_calls([
            call('dir/', os.path.join('dir/127.0.0.1', fname))
            for _, fname in TEST_ARCHIVE_CONTENTS
        ])
        self.assertEqual(make_entry_dirs.call_count, 2)
        self.assertEqual(stdout.read(), '')
        self.assertEqual(stderr.read(), '')

    def test_extract_verbose(self, stdout, stderr,
                             write, make_entry_dirs, makedirs, _):
        extract(TEST_ARCHIVE['log']['entries'], 'dir/', verbose=True)
        stdout.seek(0)
        stderr.seek(0)
        makedirs.assert_called_with('dir/', exist_ok=True)
        write.assert_has_calls([
            call(content, os.path.join('dir', fname))
            for content, fname in TEST_ARCHIVE_CONTENTS
        ])
        self.assertEqual(make_entry_dirs.call_count, 0)
        self.assertEqual(stdout.read(), TEST_ARCHIVE_VERBOSE)
        self.assertEqual(stderr.read(), '')

    def test_extract_invalid_all(self, stdout, stderr,
                                 write, make_entry_dirs, makedirs, _):
        extract(TEST_ARCHIVE_INVALID['log']['entries'],
                'dir/', verbose=True, exit_on_error=False)
        stdout.seek(0)
        stderr.seek(0)
        makedirs.assert_called_with('dir/', exist_ok=True)
        write.assert_called_with('404', 'dir/404')
        self.assertEqual(make_entry_dirs.call_count, 0)
        self.assertEqual(stdout.read(), TEST_ARCHIVE_INVALID_VERBOSE)
        self.assertNotEqual(stderr.read(), '')

    def test_extract_invalid(self, stdout, stderr,
                             write, make_entry_dirs, makedirs, _):
        output = TEST_ARCHIVE_INVALID_LIST.split('\n')
        output = output[0] + '\n'
        with self.assertRaises(ValueError):
            extract(TEST_ARCHIVE_INVALID['log']['entries'],
                    'dir/', verbose=True)
        stdout.seek(0)
        stderr.seek(0)
        makedirs.assert_called_with('dir/', exist_ok=True)
        self.assertEqual(make_entry_dirs.call_count, 0)
        self.assertEqual(write.call_count, 0)
        self.assertEqual(stdout.read(), output)
        self.assertEqual(stderr.read(), '')

    def test_extract_ioerror(self, stdout, stderr,
                             write, make_entry_dirs, makedirs, _):
        write.side_effect = IOError

        extract(TEST_ARCHIVE['log']['entries'],
                'dir/', verbose=True, exit_on_error=False)

        makedirs.assert_called_with('dir/', exist_ok=True)
        stdout.seek(0)
        stderr.seek(0)
        write.assert_has_calls([
            call(content, os.path.join('dir', fname))
            for content, fname in TEST_ARCHIVE_CONTENTS
        ])
        self.assertEqual(make_entry_dirs.call_count, 0)
        self.assertEqual(stdout.read(), TEST_ARCHIVE_VERBOSE)
        self.assertNotEqual(stderr.read(), '')

        with self.assertRaises(IOError):
            extract(TEST_ARCHIVE['log']['entries'], 'dir/', verbose=True)

    def test_extract_list(self, stdout, stderr,
                          write, make_entry_dirs, makedirs, _):
        extract(TEST_ARCHIVE['log']['entries'], None)
        stdout.seek(0)
        stderr.seek(0)
        self.assertEqual(makedirs.call_count, 0)
        self.assertEqual(write.call_count, 0)
        self.assertEqual(make_entry_dirs.call_count, 0)
        self.assertEqual(stdout.read(), TEST_ARCHIVE_LIST)
        self.assertEqual(stderr.read(), '')

    def test_extract_list_invalid(self, stdout, stderr,
                                  write, make_entry_dirs, makedirs, _):
        extract(TEST_ARCHIVE_INVALID['log']['entries'], None)
        stdout.seek(0)
        stderr.seek(0)
        self.assertEqual(stdout.read(), TEST_ARCHIVE_INVALID_LIST)
        self.assertEqual(stderr.read(), '')
        self.assertEqual(makedirs.call_count, 0)
        self.assertEqual(write.call_count, 0)
        self.assertEqual(make_entry_dirs.call_count, 0)
        #extract(TEST_ARCHIVE_INVALID['log']['entries'], None, False, False)



TEST_ARCHIVE = {
    'log': {
        'entries': [
            {
                'request': {
                    'method': 'GET',
                    'url': 'https://127.0.0.1/',
                },
                'response': {
                    'status': 200,
                    'statusText': 'OK',
                    'content': {
                        'mimeType': 'text/plain',
                        'size': 4,
                        'text': 'test'
                    }
                }
            },
            {
                'request': {
                    'method': 'GET',
                    'url': 'https://127.0.0.1/dir/',
                },
                'response': {
                    'status': 200,
                    'statusText': 'OK',
                    'content': {
                        'mimeType': "text/plain",
                        'size': 8,
                        'encoding': 'base64',
                        'text': 'dGVzdDIK'
                    }
                }
            },
            {
                'request': {
                    'method': 'GET',
                    'url': 'https://127.0.0.1/404',
                },
                'response': {
                    'status': 404,
                    'statusText': 'Not Found',
                    'content': {
                        'mimeType': "text/plain",
                        'size': 0,
                        'text': ''
                    }
                }
            }
        ]
    }
}

TEST_ARCHIVE_INVALID = {
    'log': {
        'entries': [
            {
                'response': {
                    'status': 200,
                    'statusText': 'OK',
                    'content': {
                        'mimeType': 'text/plain',
                        'size': 4,
                        'text': 'test'
                    }
                }
            },
            {},
            {
                'request': {
                    'method': 'GET',
                    'url': 'https://127.0.0.1/404',
                },
                'response': {
                    'status': 404,
                    'statusText': 'Not Found',
                    'content': {
                        'mimeType': "text/plain",
                        'size': 3,
                        'text': '404'
                    }
                }
            }
        ]
    }
}

TEST_ARCHIVE_CONTENTS = [
    ('test', 'index.html'),
    (b'test2\n', 'dir')
]

TEST_ARCHIVE_VERBOSE = '''GET https://127.0.0.1/ -> 200 OK text/plain 4B
\t----> dir/index.html
GET https://127.0.0.1/dir/ -> 200 OK text/plain 8B
\t----> dir/dir
GET https://127.0.0.1/404 -> 404 Not Found text/plain 0B
\t----> <no content>
'''

TEST_ARCHIVE_INVALID_VERBOSE = '''<no method> <no url> -> 200 OK text/plain 4B
<no method> <no url> -> <no status> <no status text> <no mime type> <invalid size>
\t----> <no content>
GET https://127.0.0.1/404 -> 404 Not Found text/plain 3B
\t----> dir/404
'''

TEST_ARCHIVE_LIST = '''GET https://127.0.0.1/ -> 200 OK text/plain 4B
GET https://127.0.0.1/dir/ -> 200 OK text/plain 8B
GET https://127.0.0.1/404 -> 404 Not Found text/plain 0B
'''

TEST_ARCHIVE_INVALID_LIST = '''<no method> <no url> -> 200 OK text/plain 4B
<no method> <no url> -> <no status> <no status text> <no mime type> <invalid size>
GET https://127.0.0.1/404 -> 404 Not Found text/plain 3B
'''



# pylint:disable=too-many-arguments

from unittest import TestCase
from unittest.mock import patch, mock_open
from io import StringIO

from har_extractor import main


@patch('sys.stderr', new_callable=StringIO)
@patch('shutil.rmtree')
@patch('builtins.open', new_callable=mock_open)
@patch('har_extractor.get_out_dir', return_value='outdir')
@patch('har_extractor.get_entries', return_value='entries')
@patch('har_extractor.extract')
class TestMain(TestCase):
    @patch('sys.exit')
    def test_error(self, exit_, extract, *_):
        self.assertEqual(main(['-v']), 1)
        exit_.assert_called_with(2)
        self.assertEqual(extract.call_count, 0)

    def test_default(self, extract, get_entries,
                     get_out_dir, open_, rmtree, stderr):
        handle = open_()
        self.assertEqual(main(['file']), 0)
        stderr.seek(0)
        self.assertEqual(stderr.read(), '')
        self.assertEqual(rmtree.call_count, 0)
        get_out_dir.assert_called_with(None, 'file.d')
        open_.assert_called_with('file', 'rb')
        get_entries.assert_called_with(handle, False)
        extract.assert_called_with('entries', 'outdir', True, True, False)

    def test_list(self, extract, get_entries,
                  get_out_dir, open_, rmtree, stderr):
        handle = open_()
        self.assertEqual(main(['-l', 'file']), 0)
        stderr.seek(0)
        self.assertEqual(stderr.read(), '')
        self.assertEqual(rmtree.call_count, 0)
        self.assertEqual(get_out_dir.call_count, 0)
        open_.assert_called_with('file', 'rb')
        get_entries.assert_called_with(handle, False)
        extract.assert_called_with('entries', None, True, True, False)

    def test_args(self, extract, get_entries,
                  get_out_dir, open_, rmtree, stderr):
        handle = open_()
        self.assertEqual(
            main(['-v', '-i', '-d', '-s', '-o', 'dir', 'file']),
            0
        )
        stderr.seek(0)
        self.assertEqual(stderr.read(), '')
        self.assertEqual(rmtree.call_count, 0)
        get_out_dir.assert_called_with('dir', 'file.d')
        open_.assert_called_with('file', 'rb')
        get_entries.assert_called_with(handle, True)
        extract.assert_called_with('entries', 'outdir', True, True, True)

        self.assertEqual(
            main(['-nv', '-ni', '-nd', '-ns', '-o', 'dir', 'file']),
            0
        )
        stderr.seek(0)
        self.assertEqual(stderr.read(), '')
        self.assertEqual(rmtree.call_count, 0)
        get_out_dir.assert_called_with('dir', 'file.d')
        open_.assert_called_with('file', 'rb')
        get_entries.assert_called_with(handle, False)
        extract.assert_called_with('entries', 'outdir', False, False, False)

    def test_getdir_error(self, extract, get_entries,
                          get_out_dir, open_, rmtree, stderr):
        get_out_dir.side_effect = ValueError('value error')
        self.assertEqual(main(['-v', 'file']), 1)
        stderr.seek(0)
        self.assertEqual(stderr.read(), 'value error\n')
        self.assertEqual(rmtree.call_count, 0)
        get_out_dir.assert_called_with(None, 'file.d')
        self.assertEqual(open_.call_count, 0)
        self.assertEqual(get_entries.call_count, 0)
        self.assertEqual(extract.call_count, 0)

    def test_extract_error(self, extract, get_entries,
                           get_out_dir, open_, rmtree, stderr):
        handle = open_()
        extract.side_effect = IOError('io error')
        self.assertEqual(main(['file']), 1)
        stderr.seek(0)
        self.assertEqual(stderr.read(), 'io error\n')
        self.assertEqual(rmtree.call_count, 0)
        get_out_dir.assert_called_with(None, 'file.d')
        open_.assert_called_with('file', 'rb')
        get_entries.assert_called_with(handle, False)
        extract.assert_called_with('entries', 'outdir', True, True, False)

    def test_extract_error_strict(self, extract, get_entries,
                                  get_out_dir, open_, rmtree, stderr):
        handle = open_()
        extract.side_effect = IOError('io error')
        self.assertEqual(main(['-s', 'file']), 1)
        stderr.seek(0)
        self.assertEqual(stderr.read(), 'io error\n')
        rmtree.assert_called_with('outdir')
        get_out_dir.assert_called_with(None, 'file.d')
        open_.assert_called_with('file', 'rb')
        get_entries.assert_called_with(handle, False)
        extract.assert_called_with('entries', 'outdir', True, True, True)



#!/usr/bin/env python3

import os
from unittest import TestLoader
from setuptools import setup

def tests():
    return TestLoader().discover('tests')

BASE_DIR = os.path.abspath(os.path.dirname(__file__))
try:
    with open(os.path.join(BASE_DIR, 'README.rst')) as fp:
        README = fp.read()
except IOError:
    README = ''

setup(
    name='har-extractor',
    version='1.0.1',
    description='HTTP Archive extractor',
    long_description=README,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: System :: Archiving',
        'Topic :: Utilities'
    ],
    keywords='har http archive extractor',
    url='https://github.com/dead-beef/har-extractor',
    author='dead-beef',
    author_email='contact@dead-beef.tk',
    license='MIT',
    py_modules=['har_extractor'],
    entry_points={
        'console_scripts': [
            'har-extractor=har_extractor:main'
        ]
    },
    test_suite='setup.tests',
    install_requires=['ijson'],
    extras_require={
        'dev': [
            'coverage',
            'twine>=1.8.1',
            'wheel'
        ]
    },
    python_requires='>=3',
    include_package_data=True,
    zip_safe=False
)



#!/usr/bin/env python3

from argparse import ArgumentParser
from urllib.parse import urlparse
from base64 import b64decode
from itertools import count

import os
import sys
import json
import shutil

try:
    import ijson.backends.yajl2_cffi as ijson
except ImportError:
    try:
        import ijson.backends.yajl2 as ijson
    except ImportError:
        try:
            import ijson
        except ImportError:
            ijson = None


__appname__ = 'har-extractor'
__version__ = '1.0.1'

NAME_VERSION = '%s %s' % (__appname__, __version__)
SIZE_UNITS = 'BKMGT'


def format_size(size):
    if size < 0:
        return '<invalid size>'
    unit_name = 'B'
    unit_value = 1
    for unit_name in SIZE_UNITS:
        if size < 1024 * unit_value:
            break
        unit_value *= 1024
    if size % unit_value == 0:
        return '%d%s' % (size // unit_value, unit_name)
    return '%.2f%s' % (size / unit_value, unit_name)

def get_unused_name(path):
    if not os.path.exists(path):
        return path
    name, ext = os.path.splitext(path)
    i = 1
    while True:
        path = '%s.%d%s' % (name, i, ext)
        if not os.path.exists(path):
            return path
        i += 1

def write(content, fname):
    if isinstance(content, bytes):
        mode = 'wb'
    else:
        mode = 'w'
    with open(fname, mode) as fp:
        fp.write(content)

def format_entry(entry):
    request = entry.get('request', {})
    response = entry.get('response', {})
    content = response.get('content', {})
    return '%s %s -> %s %s %s %s' % (
        request.get('method', '<no method>'),
        request.get('url', '<no url>'),
        response.get('status', '<no status>'),
        response.get('statusText', '<no status text>'),
        content.get('mimeType', '<no mime type>'),
        format_size(content.get('size', -1))
    )

def get_entry_content(entry):
    try:
        content = entry['response']['content']
    except KeyError:
        return None
    try:
        text = content['text']
        if not text:
            return None
    except KeyError:
        return None

    try:
        if content['encoding'] == 'base64':
            text = b64decode(text)
        else:
            raise ValueError(
                '\tUnknown content encoding: "%s"' % content['encoding']
            )
    except KeyError:
        pass

    return text

def get_entry_path(entry, subdirs=False):
    try:
        url = urlparse(entry['request']['url'])
    except KeyError:
        raise ValueError('Invalid entry: missing request URL: %s' % repr(entry))

    fname = url.path.strip('/')
    if fname == '':
        fname = 'index.html'

    if subdirs:
        return os.path.join(url.netloc, fname)
    return os.path.basename(fname)

def get_entries(fp, iterative=True):
    if fp is sys.stdin:
        iterative = True
        fp = fp.buffer

    if ijson is None or not iterative:
        data = fp.read()
        if isinstance(data, bytes):
            data = data.decode('utf-8')
        data = json.loads(data)
        return data['log']['entries']
    else:
        return ijson.items(fp, 'log.entries.item')

def get_out_dir(path, default):
    if not path:
        return default
    if os.path.exists(path):
        if not os.path.isdir(path):
            raise ValueError('"%s" is not a directory' % path)
        return os.path.join(path, default)
    return path

def dirnames(entry, root):
    path = os.path.relpath(entry, root)
    ret = []
    path = os.path.dirname(path)
    while path:
        ret.append(os.path.join(root, path))
        path = os.path.dirname(path)
    return ret

def move_files_to_dir(path, first):
    dirname, name = os.path.split(path)
    name, ext = os.path.splitext(name)
    shutil.move(first, os.path.join(path, 'index.html'))
    for i in count(1):
        fpath = os.path.join(dirname, '%s.%d%s' % (name, i, ext))
        if not os.path.exists(fpath):
            return
        fname = 'index.%d.html' % i
        shutil.move(fpath, os.path.join(path, fname))

def make_entry_dirs(root, entry):
    try:
        os.makedirs(os.path.dirname(entry), exist_ok=True)
        return
    except OSError:
        for path in reversed(dirnames(entry, root)):
            if not os.path.exists(path):
                os.mkdir(path)
            elif not os.path.isdir(path):
                tmp = get_unused_name(path)
                shutil.move(path, tmp)
                os.mkdir(path)
                move_files_to_dir(path, tmp)


def extract(entries, outdir=None,
            subdirs=False, verbose=False, exit_on_error=True):
    if outdir is not None:
        os.makedirs(outdir, exist_ok=True)

    for entry in entries:
        try:
            if verbose or outdir is None:
                print(format_entry(entry))

            if outdir is None:
                continue

            content = get_entry_content(entry)
            if content is None:
                if verbose:
                    print('\t----> <no content>')
                continue

            fname = get_entry_path(entry, subdirs)
            fname = os.path.join(outdir, fname)
            fname = get_unused_name(fname)
            if verbose:
                print('\t---->', fname)

            try:
                if subdirs:
                    make_entry_dirs(outdir, fname)
                write(content, fname)
            except (OSError, IOError) as err:
                msg = 'Could not write "%s": %s' % (fname, repr(err))
                if exit_on_error:
                    raise IOError(msg)
                else:
                    print(msg, file=sys.stderr)
        except (KeyError, ValueError) as err:
            msg = 'Invalid entry: %s: %s' % (repr(entry), repr(err))
            if exit_on_error:
                raise ValueError(msg)
            else:
                print(msg, file=sys.stderr)


def main(args=None):
    parser = ArgumentParser(args)

    parser.add_argument('file', metavar='FILE', help='HAR file')

    parser.add_argument('-V', '--version',
                        action='version', version=NAME_VERSION)

    parser.add_argument('-l', '--list', action='store_true',
                        help='list the contents of input file')

    parser.add_argument('-o', '--output',
                        metavar='DIRECTORY', default=None,
                        help='set output directory (default: ./<filename>.d)')

    parser.add_argument('-v', '--verbose',
                        dest='verbose',
                        action='store_true',
                        help='turn on verbose output (default)')

    parser.add_argument('-nv', '--no-verbose',
                        dest='verbose',
                        action='store_false',
                        help='turn off verbose output')

    parser.add_argument('-i', '--iterative',
                        dest='iterative',
                        action='store_true',
                        help='use iterative json parser')

    parser.add_argument('-ni', '--no-iterative',
                        dest='iterative',
                        action='store_false',
                        help='do not use iterative json parser (default)')

    parser.add_argument('-s', '--strict',
                        dest='strict',
                        action='store_true',
                        help='exit and delete extracted data after first error')

    parser.add_argument('-ns', '--no-strict',
                        dest='strict',
                        action='store_false',
                        help='ignore errors (default)')

    parser.add_argument('-d', '--directories',
                        dest='directories',
                        action='store_true',
                        help='create url directories (default)')

    parser.add_argument('-nd', '--no-directories',
                        dest='directories',
                        action='store_false',
                        help='do not create url directories')

    parser.set_defaults(
        iterative=False,
        directories=True,
        strict=False,
        verbose=True
    )

    if args is not None:
        args = parser.parse_args(args)
    else:
        args = parser.parse_args()

    if args.file is None:
        return 1

    if args.list:
        outdir = None
    else:
        try:
            outdir = get_out_dir(args.output,
                                 os.path.basename(args.file) + '.d')
        except ValueError as err:
            print(err, file=sys.stderr)
            return 1

    try:
        with open(args.file, 'rb') as fp:
            entries = get_entries(fp, args.iterative)
            extract(entries, outdir,
                    args.directories, args.verbose, args.strict)
    except (ValueError, IOError) as err:
        if args.strict:
            shutil.rmtree(outdir)
        print(err, file=sys.stderr)
        return 1

    return 0


if __name__ == '__main__':
    sys.exit(main())
